/**
 * 
 */
package com.wieseke.cptk.common.util;

import java.lang.reflect.Array;
import java.util.Iterator;
import java.util.List;

/**
 * @author Nicolas Wieseke
 * 
 */
public class CombinationIterator<T> implements Iterator<T[]> {

	/**
	 * an array of lists containing the objects
	 */
	private List<T>[] array;
	/**
	 * pointers to the objects for the current combination
	 */
	private int[] pointer;
	/**
	 * the number of objects in each combination
	 */
	private int size;
	/**
	 * the number of combinations left
	 */
	private long combinations;
	private Class<?> usedClass;

	/**
	 * default constructor
	 * 
	 * @param array
	 *            the array of lists
	 */
	public CombinationIterator(List<T>[] array, Class<?> usedClass) {
		this.size = array.length;
		this.array = array;
		this.usedClass = usedClass;
		this.pointer = new int[size];
		this.pointer[size - 1] = -1;
		this.combinations = 1;
		for (int index = 0; index < this.size; index++) {
			combinations *= this.array[index].size() > 0 ? this.array[index].size() : 1;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Iterator#hasNext()
	 */
	
	public boolean hasNext() {
		return combinations > 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Iterator#next()
	 */
	@SuppressWarnings("unchecked")
	
	public T[] next() {
		if (combinations == 0) {
			return null;
		}
		T[] next = (T[]) Array.newInstance(usedClass, size);
		pointer[size - 1]++;
		for (int index = size - 1; index >= 0; index--) {
			if (pointer[index] >= array[index].size()) {
				pointer[index] = 0;
				if (index > 0)
					pointer[index - 1]++;
			}
			if (array[index].size() > 0) {
				next[index] = array[index].get(pointer[index]);
			} else
				next[index] = null;
		}
		combinations--;
		return next;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Iterator#remove()
	 */
	
	public void remove() {
		this.next();
	}

}
