/**
 * 
 */
package com.wieseke.cptk.common.wizards;

import java.awt.Dimension;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStreamWriter;

import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;

import com.wieseke.cptk.common.log.CophylogenyLogger;
import com.wieseke.cptk.common.viewer.CophylogenyViewer;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ExportSvgFileWizardPage extends WizardNewFileCreationPage {

	private CophylogenyViewer viewer;

	public ExportSvgFileWizardPage(CophylogenyViewer viewer,
			IStructuredSelection selection) {
		super("WizardExportSvgFileCreationPage", selection);
		setTitle("Select SVG File");
		setDescription("Specify a file to where the svg data should be saved.");
		setFileExtension("svg");
		this.viewer = viewer;
	}

	protected InputStream getInitialContents() {
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		try {

			// Get a DOMImplementation.
			DOMImplementation domImpl = GenericDOMImplementation
					.getDOMImplementation();

			// Create an instance of org.w3c.dom.Document.
			Document document = domImpl.createDocument(
					"http://www.w3.org/2000/svg", "svg", null);

			// Create an instance of the SVG Generator.
			SVGGraphics2D svgGenerator = new SVGGraphics2D(document);

			// set the size
			svgGenerator.setSVGCanvasSize(new Dimension(viewer.normalize(viewer
					.getMinWidth()), viewer.normalize(viewer.getMinHeight())));

			// Ask the cophylogeny to render into the SVG Graphics2D
			// implementation.
			viewer.paintContent(svgGenerator);

			// Finally, stream out SVG to the standard output using
			OutputStreamWriter osw = new OutputStreamWriter(stream);
			svgGenerator.stream(osw, true);
			osw.close();

		} catch (Exception e) {
			CophylogenyLogger.logError("Could not convert file to SVG.", e);
		}

		return new ByteArrayInputStream(stream.toByteArray());
	}
}