/**
 * 
 */
package com.wieseke.cptk.input.dao;

import java.util.ArrayList;
import java.util.List;

import com.wieseke.cptk.common.api.IPositionParasiteNode;
import com.wieseke.cptk.common.dao.ParasiteRank;

/**
 * @author Nicolas Wieseke
 * 
 */
public class InputParasiteNode extends InputNode implements
		IPositionParasiteNode {

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.dao.PositionNode#getParent()
	 */
	
	public InputParasiteNode getParent() {
		return (InputParasiteNode) super.getParent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.input.dao.InputNode#getChildren()
	 */
	@SuppressWarnings("unchecked")
	public List<InputParasiteNode> getChildren() {
		return (List<InputParasiteNode>) super.getChildren();
	}

	/**
	 * timezone information for the parasite node
	 */
	private ParasiteRank rank;

	/**
	 * the ids of the host nodes to which this parasite is associated with.
	 */
	private List<InputHostNode> associations;

	/**
	 * the ids of the host nodes to which this parasite is associated with.
	 */
	private List<InputHostNode> reconstructionAssociations;

	/**
	 * default constructor
	 */
	public InputParasiteNode() {
		super();
		this.associations = new ArrayList<InputHostNode>();
		this.reconstructionAssociations = new ArrayList<InputHostNode>();
		this.rank = new ParasiteRank(0, 0);
	}

	/**
	 * @return the rank
	 */
	
	public ParasiteRank getRank() {
		return rank;
	}

	/**
	 * @param rank
	 *            the rank to set
	 */
	
	public void setRank(ParasiteRank rank) {
		this.rank = rank;
	}

	/**
	 * @return the associations
	 */
	
	public List<InputHostNode> getAssociations() {
		return associations;
	}

	/**
	 * @param associations
	 *            the associations to set
	 */
	public void setAssociations(List<InputHostNode> associations) {
		this.associations = associations;
	}

	/**
	 * adds a host node to the list of associations
	 * 
	 * @param hostNode
	 *            the HostNode to add to the associations list
	 * @return the return value of the arraylist add operation
	 */
	public boolean addAssociations(InputHostNode hostNode) {
		return this.associations.add(hostNode);

	}

	/**
	 * @return the associations
	 */

	public List<InputHostNode> getReconstructionAssociations() {
		return reconstructionAssociations;
	}

	/**
	 * @param associations
	 *            the associations to set
	 */
	public void setReconstructionAssociations(
			List<InputHostNode> reconstructionAssociations) {
		this.reconstructionAssociations = reconstructionAssociations;
	}

	/**
	 * adds a host node to the list of associations
	 * 
	 * @param hostNode
	 *            the HostNode to add to the associations list
	 * @return the return value of the arraylist add operation
	 */
	public boolean addReconstructionAssociations(InputHostNode hostNode) {
		return this.reconstructionAssociations.add(hostNode);

	}

	public ParasiteRank getParentRank() {
		if (getParent() == null)
			return new ParasiteRank(0, 0);
		else
			return getParent().getRank();
	}

	public ParasiteRank getMinChildRank() {
		if (isLeaf())
			return new ParasiteRank(Integer.MAX_VALUE, Integer.MAX_VALUE);
		else {
			ParasiteRank rank = new ParasiteRank(Integer.MAX_VALUE,
					Integer.MAX_VALUE);
			for (InputParasiteNode node : getChildren()) {
				if (rank.getRankFrom() > node.getRank().getRankFrom())
					rank.setRankFrom(node.getRank().getRankFrom());
				if (rank.getRankTo() > node.getRank().getRankTo())
					rank.setRankTo(node.getRank().getRankTo());
			}
			return rank;
		}
	}

	
	public String getToolTipText() {
		return getLabel() + " (" + getRank().toString() + ")";
	}

	
	public String getRankText() {
		return rank.toString();
	}

}
