/**
 * 
 */
package com.wieseke.cptk.output.action;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.output.constants.OutputActionConstants;
import com.wieseke.cptk.output.operation.EditChangeLabelOperation;
import com.wieseke.cptk.output.viewer.OutputCophylogenyViewer;
import com.wieseke.cptk.output.viewer.OutputNodeControl;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditChangeLabelAction extends OutputAction {

	/**
	 * default constructor
	 */
	public EditChangeLabelAction() {
		super(OutputActionConstants.CHANGE_LABEL_TEXT);
	}

	/**
	 * constructor
	 * 
	 * @param viewer
	 * @param node
	 */
	public EditChangeLabelAction(OutputCophylogenyViewer viewer,
			OutputNodeControl node) {
		super(OutputActionConstants.CHANGE_LABEL_TEXT);
		this.viewer = viewer;
		this.node = node;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	
	public void run() {
		if (this.node != null) {
			EditChangeLabelOperation operation = new EditChangeLabelOperation(
					getViewer(), this.node.getNode());
			operation.addContext(this.viewer.getUndoContext());
			try {
				PlatformUI.getWorkbench().getOperationSupport()
						.getOperationHistory().execute(operation, null, null);
			} catch (ExecutionException e) {
				MessageDialog.openError(this.viewer.getShell(),
						"Change Label Error",
						"Exception while changing the label of node "
								+ this.node.getNode().getLabel() + ": "
								+ e.getMessage());
			}
		}
	}
}