/**
 * 
 */
package com.wieseke.cptk.output.viewer;

import org.eclipse.swt.graphics.Point;

import com.wieseke.cptk.common.api.IPositionNode;
import com.wieseke.cptk.common.api.ITreeArrangement;
import com.wieseke.cptk.output.dao.OutputCophylogeny;

/**
 * @author Nicolas Wieseke
 * 
 */
public class OutputTreeArrangement implements ITreeArrangement {

	public static final int ARRANGEMENT_HORIZONTAL = 0;
	public static final int ARRANGEMENT_VERTICAL = 1;

	public static final int POSITION_FIRST_ROOT_SECOND_CHILDREN = 0;
	public static final int POSITION_FIRST_CHILDREN_SECOND_ROOT = 1;

	private Point treeRootPosition;
	private Point treeFirstLeafPosition;
	private Point treeLastLeafPosition;

	private int arrangementFlag;
	private int positionFlag;
	private boolean treeFlipped;

	public static int getArrangement(IPositionNode root) {
		IPositionNode firstLeaf = root;
		while (firstLeaf.getChildren().size() != 0) {
			firstLeaf = firstLeaf.getChildren().get(0);
		}
		Point firstLeafPosition = new Point(firstLeaf.getPosX(), firstLeaf.getPosY());
		IPositionNode lastLeaf = root;
		while (lastLeaf.getChildren().size() != 0) {
			lastLeaf = lastLeaf.getChildren().get(lastLeaf.getChildren().size() - 1);
		}
		Point lastLeafPosition = new Point(lastLeaf.getPosX(), lastLeaf.getPosY());

		int horizontalDistance = Math.abs(firstLeafPosition.x - lastLeafPosition.x);
		int verticalDistance = Math.abs(firstLeafPosition.y - lastLeafPosition.y);

		if (verticalDistance >= horizontalDistance) {
			return ARRANGEMENT_HORIZONTAL;
		} else {
			return ARRANGEMENT_VERTICAL;
		}

	}

	public OutputTreeArrangement(OutputCophylogeny cophylogeny) {
		IPositionNode hostRoot = cophylogeny.getHostRoot();
		this.treeRootPosition = new Point(hostRoot.getPosX(), hostRoot.getPosY());
		IPositionNode firstHostLeaf = hostRoot;
		while (firstHostLeaf.getChildren().size() != 0) {
			firstHostLeaf = firstHostLeaf.getChildren().get(0);
		}
		this.treeFirstLeafPosition = new Point(firstHostLeaf.getPosX(), firstHostLeaf.getPosY());
		IPositionNode lastHostLeaf = hostRoot;
		while (lastHostLeaf.getChildren().size() != 0) {
			lastHostLeaf = lastHostLeaf.getChildren().get(lastHostLeaf.getChildren().size() - 1);
		}
		this.treeLastLeafPosition = new Point(lastHostLeaf.getPosX(), lastHostLeaf.getPosY());

		int horizontalDistance = Math.abs(this.treeFirstLeafPosition.x - this.treeLastLeafPosition.x);
		int verticalDistance = Math.abs(this.treeFirstLeafPosition.y - this.treeLastLeafPosition.y);
		if (verticalDistance >= horizontalDistance) {
			this.arrangementFlag = ARRANGEMENT_HORIZONTAL;
			if (this.treeRootPosition.x <= this.treeFirstLeafPosition.x
					&& this.treeRootPosition.x <= this.treeLastLeafPosition.x) {
				this.positionFlag = POSITION_FIRST_ROOT_SECOND_CHILDREN;
				if (this.treeFirstLeafPosition.y > this.treeLastLeafPosition.y) {
					this.treeFlipped = true;
				} else {
					this.treeFlipped = false;
				}
			} else {
				this.positionFlag = POSITION_FIRST_CHILDREN_SECOND_ROOT;
				if (this.treeFirstLeafPosition.y >= this.treeLastLeafPosition.y) {
					this.treeFlipped = false;
				} else {
					this.treeFlipped = true;
				}
			}
		} else {
			this.arrangementFlag = ARRANGEMENT_VERTICAL;
			if (this.treeRootPosition.y <= this.treeFirstLeafPosition.y
					&& this.treeRootPosition.y <= this.treeLastLeafPosition.y) {
				this.positionFlag = POSITION_FIRST_ROOT_SECOND_CHILDREN;
				if (this.treeFirstLeafPosition.x >= this.treeLastLeafPosition.x) {
					this.treeFlipped = false;
				} else {
					this.treeFlipped = true;
				}
			} else {
				this.positionFlag = POSITION_FIRST_CHILDREN_SECOND_ROOT;
				if (this.treeFirstLeafPosition.x > this.treeLastLeafPosition.x) {
					this.treeFlipped = true;
				} else {
					this.treeFlipped = false;
				}
			}
		}
	}

	/**
	 * @return the hostRootPosition
	 */
	public Point getTreeRootPosition(int tree) {
		return treeRootPosition;
	}

	/**
	 * @return the hostFlipped
	 */
	public boolean isTreeFlipped(int tree) {
		return treeFlipped;
	}

	/**
	 * @return the arrangementFlag
	 */
	public int getArrangementFlag() {
		return arrangementFlag;
	}

	/**
	 * @return the positionFlag
	 */
	public int getPositionFlag() {
		return positionFlag;
	}

}
